/* 
 * Core Interface for mks32 			
 *
 * Copyright (C) Rainer Link, 2001-2002
 *                            OpenAntiVirus.org <rainer@openantivirus.org>
 *		 Krzysztof Drewicz <hunter@pld.org.pl>, 2002
 *
 * Credits to W. Richard Stevens - RIP
 * 
 * This software is licensed under the GNU General Public License (GPL)
 * See: http://www.gnu.org/copyleft/gpl.html
 *
*/

#include <includes.h>

#include "vscan-message.h"
#include "vscan-functions.h"
#include "vscan-mksd_core.h"


extern BOOL verbose_file_logging;
extern BOOL send_warning_message;
 
/* initialise socket to mks32
   returns -1 on error or the socket descriptor */
int vscan_mksd_init(void)
{
	return 0;
}

/*
  If virus is found, logs the filename/virusname into syslog
*/
void vscan_mksd_log_virus(char *infected_file, char *result, char* client_ip)
{
        vscan_syslog_alert("ALERT - Scan result: '%s' infected with virus '%s', client: '%s'", infected_file, result, client_ip);
	if ( send_warning_message )
		vscan_send_warning_message(infected_file, result, client_ip);
        
}



/*
  Scans a file (*FILE*, not a directory - keep that in mind) for a virus
  Expects socket descriptor and file name to scan for
  Returns -2 on a minor error,  -1 on error, 0 if no virus was found, 
  1 if a virus was found 
*/
int vscan_mksd_scanfile(int sockfd, char *scan_file, char* client_ip)
{
	char recv[MAXBUFF];
	int result,len;

	bzero(recv,sizeof(recv));

	if ( verbose_file_logging )
	        vscan_syslog("INFO: Scanning file : '%s'", scan_file);

	/* what happens if scan_file is very large? */
	result = mksc_check_filename (scan_file, recv);
	vscan_syslog("%s", recv+4);

	if ( result==1 ) {
		/* virus found */
		vscan_mksd_log_virus(scan_file, recv, client_ip);
		return 1;
	}  	else if ( result==127  ) {
		/* ERROR */
		if ( verbose_file_logging )
			vscan_syslog("ERROR: file %s not found or not readable or deamon not running", scan_file);
			/* FIXME: daemon not running doesn't actually qualify as minor error ... */
		return -2;
	} else if (  result == 0 ) {
		/* OK */
		if ( verbose_file_logging )
			vscan_syslog("INFO: file %s is clean (%s)", scan_file, recv);
        } else {
		/* unknown response */
		vscan_syslog("ERROR: unknown response from mksd while scanning %s! (%s)", scan_file,recv);
		return -2;
	}	

	return 0;
	
}

/*
  close socket
*/
void vscan_mksd_end(int sockfd)
{
}

